<?php

namespace yifang\model\relation;

use yifang\model\Relation;
use Closure;
use yifang\utils\Str;
use yifang\db\BaseQuery as Query;
use yifang\model\Model;
use yifang\model\Pivot;
use yifang\db\Raw;
use Exception;

/**
 * 远程一对一关联类
 */
class HasOneThrough extends HasManyThrough
{

    /**
     * 延迟获取关联数据
     * @return Model
     */
    public function getRelation(array $subRelation = [], Closure $closure = null)
    {
        if ($closure) {
            $closure($this->getClosureType($closure));
        }
        $this->baseQuery();
        $relationModel = $this->query->relation($subRelation)->find();
        if ($relationModel) {
            $relationModel->setParent(clone $this->parent);
        } else {
            $relationModel = $this->getDefaultModel();
        }
        return $relationModel;
    }

    /**
     * 预载入关联查询（数据集）
     * @return void
     */
    public function eagerlyResultSet(array &$resultSet,  $relation, array $subRelation = [], Closure $closure = null, array $cache = [])
    {
        $localKey   = $this->localKey;
        $foreignKey = $this->foreignKey;
        $range = [];
        foreach ($resultSet as $result) {
            // 获取关联外键列表
            if (isset($result->$localKey)) {
                $range[] = $result->$localKey;
            }
        }
        if (!empty($range)) {
            $this->query->removeWhereField($foreignKey);
            $data = $this->eagerlyWhere([
                [$this->foreignKey, 'in', $range],
            ], $foreignKey, $subRelation, $closure, $cache);
            // 关联数据封装
            foreach ($resultSet as $result) {
                // 关联模型
                if (!isset($data[$result->$localKey])) {
                    $relationModel = $this->getDefaultModel();
                } else {
                    $relationModel = $data[$result->$localKey];
                    $relationModel->setParent(clone $result);
                    $relationModel->exists(true);
                }
                // 设置关联属性
                $result->setRelation($relation, $relationModel);
            }
        }
    }

    /**
     * 预载入关联查询（数据）
     * @return void
     */
    public function eagerlyResult(Model $result,  $relation, array $subRelation = [], Closure $closure = null, array $cache = [])
    {
        $localKey   = $this->localKey;
        $foreignKey = $this->foreignKey;
        $this->query->removeWhereField($foreignKey);
        $data = $this->eagerlyWhere([
            [$foreignKey, '=', $result->$localKey],
        ], $foreignKey, $subRelation, $closure, $cache);
        // 关联模型
        if (!isset($data[$result->$localKey])) {
            $relationModel = $this->getDefaultModel();
        } else {
            $relationModel = $data[$result->$localKey];
            $relationModel->setParent(clone $result);
            $relationModel->exists(true);
        }
        $result->setRelation($relation, $relationModel);
    }

    /**
     * 关联模型预查询
     * @return array
     */
    protected function eagerlyWhere(array $where,  $key, array $subRelation = [], Closure $closure = null, array $cache = [])
    {
        // 预载入关联查询 支持嵌套预载入
        $keys = $this->through->where($where)->column($this->throughPk, $this->foreignKey);
        if ($closure) {
            $closure($this->getClosureType($closure));
        }
        $list = $this->query
            ->where($this->throughKey, 'in', $keys)
            ->cache($cache[0] ?? false, $cache[1] ?? null, $cache[2] ?? null)
            ->select();
        // 组装模型数据
        $data = [];
        $keys = array_flip($keys);
        foreach ($list as $set) {
            $data[$keys[$set->{$this->throughKey}]] = $set;
        }
        return $data;
    }
}
